#!/usr/bin/perl -w
use strict;
# server_start.pl  - Start stdysrvr process, exit with error if it's running
#  Run server bg
#  If you choose to initialize the dialers we do not check to see if they are running.
#  If command line option = ALL or a number try and init the specific dialer or all listed in parmfile, NONE if no dialers.
# Cron Job Example: 08 05 * * * csh -c "setenv CFMC /cfmc/live/; setenv CFMCCFG /cfmc/live/ipcfiles/; /cfmc/live/scripts/server_start.pl ALL > $HOME/.bin/startlog"

my @errors = ();

my $usage = "$0 <ALL or Dialer # ie:01 (optional)>";

my $arrSize = @ARGV;

if( $arrSize > 1 ) {
   push(@errors,"To Many Command Line Options:\n$usage");
  }


# Check command line options
my $bg = "yes";
my $dialer = "";

if( $arrSize eq 1 ) {
     if ( $ARGV[0] eq "ALL" ) {
              $dialer = "ALL";

     } elsif ( $ARGV[0] =~ /^[+-]?\d+$/ ) {
              $dialer = $ARGV[0];
     }else{
               push(@errors,"Unknown start_server command line option:$_:\n$usage");
     }
}

my $err_array="";

# Env vars to checkL CFMC, CFMCGO, CFMCCFG

# Make sure CFMC is defined
push(@errors,&check_env_var("CFMC","directory","rx"));
&get_out(@errors) if (scalar(@errors)>0);

# CFMCCFG, CFMCGO, CFMCLOGS
#  Use defaults if not defined
#
$ENV{CFMCCFG} = "/opt/cfmc/ipc/".$ENV{CFMCACCOUNT}."/" if (! $ENV{CFMCCFG});
$ENV{CFMCGO} = $ENV{CFMC}."go/" if (! $ENV{CFMCGO});
$ENV{CONTROL} = $ENV{CFMC}."control/" if (! $ENV{CONTROL});

# readable/executable directories
my @env_vars_rx = ("CFMCCFG", "CFMCGO");
foreach my $env_rx(@env_vars_rx) {
  push(@errors,&check_env_var($env_rx,"directory","rx"));
}

# executable programs
my $stdysrvr=$ENV{CFMCGO}."stdysrvr";
my $server=$ENV{CFMCGO}."server";
my $survsupr=$ENV{CFMCGO}."survsupr";
my $makecfg=$ENV{CFMCGO}."makecfg";
my @progs = ($stdysrvr,$server,$survsupr,$makecfg);

push(@errors,&check_ex(@progs));
&get_out(@errors) if (scalar(@errors)>0);

print "Check for running study server\n";
my $srvr_pid = &get_ps_id($stdysrvr);

# Check for server ldev
my $srvr_ldev=0;
# make sure cfg files exist
my $acctcfg = <$ENV{CFMCCFG}accounts*.cfg>;
my $dashcfg = <$ENV{CFMCCFG}dash*.cfg>;
my $statcfg = <$ENV{CFMCCFG}stat*.cfg>;
my $stdycfg = <$ENV{CFMCCFG}stdy*.cfg>;

if ($acctcfg and $dashcfg and $statcfg and $stdycfg) {
  ( $srvr_ldev, $err_array ) = &check_makecfg($makecfg,$srvr_ldev);
}

if ($srvr_pid) {
  if ( $srvr_ldev > 0 ) {
    push(@errors,"Study Server running at ldev #$srvr_ldev as process #$srvr_pid");
  } else {
    push(@errors,"Study Server $stdysrvr running as process #$srvr_pid but cannot be found in $ENV{CFMCCFG} environment, something is wrong");
  }
}
else {
  if ( $srvr_ldev > 0 ) {
    push(@errors,"Study Server $stdysrvr process cannot be found but $ENV{CFMCCFG} environment shows it as using ldev #$srvr_ldev, something is wrong");
  } else {
    print "No Study Server detected\n";
  }
}

&get_out(@errors) if (scalar(@errors)>0);


#---------------------------------------------
print "Run $ENV{CFMCGO}server bg\n";

my $srvr_out = `$server bg 2>&1`;
if ($?>0) {
  push(@errors,"Cannot start $stdysrvr process ($!):\n$srvr_out\n");
  &get_out(@errors);
}

#---------------------------------------------
print "Run $survsupr to initialize dialer\n";
# Find LDEV for survsupr process starting at 9991
my $init_ldev = "9991"; # Initialize ldev to check for availability
($ENV{LDEV},$ENV{SOCKET},$err_array) = &nextldev($makecfg,$init_ldev);
push(@errors,@{$err_array});
&get_out(@errors) if (scalar(@errors)>0);

# Run $ENV{CFMCGO}survsupr

if ( $dialer eq "ALL" ) {
    &dialer_all();
}

if ( $dialer eq "NONE") {

}

if ( $dialer =~ /^[+-]?\d+$/ ){
    print "~$dialer~";
    #&dialer_specific($dialer);
}



exit(0);

##################################################
sub dialer_all {
   my $parmfile = sprintf "%s/parmfile", $ENV{CONTROL};
   my @parmfile = read_file($parmfile);

   my @dialer = grep { m/^\s*dialer\d+\s*:/ } @parmfile;

   foreach (@dialer) {
      my ($kw, $val) = split(/\s*:\s*/, $_);

      my $num = ($kw =~ m/^\s*dialer(\d++)/);

      my $suprspec = "supr$$.spx";
      open (SPEC,">$suprspec") or push(@errors, "Cannot open $suprspec for spec input ($!)");
        &get_out(@errors) if (scalar(@errors)>0);
        print SPEC ">batchjob\n";
        print SPEC "server:dialer:$num init\n";
        print SPEC "quit\n";
      close (SPEC);
      my $supr_out = `$survsupr $suprspec con 2>&1`;
        if ($?>0) {
        push(@errors,"Problem running $survsupr to initialize dialer ($!)\n$supr_out\n");
        &get_out(@errors);
        } else {
        print "Dialer $num Initialized\n";
        unlink $suprspec;
        }

   }

   return;
}

sub dialer_specific {
        my $num = ($_[0]);

        my $suprspec = "supr$$.spx";
              open (SPEC,">$suprspec") or push(@errors, "Cannot open $suprspec for spec input ($!)");
                &get_out(@errors) if (scalar(@errors)>0);
                print SPEC ">batchjob\n";
                print SPEC "server:dialer:$num init\n";
                print SPEC "quit\n";
              close (SPEC);
              my $supr_out = `$survsupr $suprspec con 2>&1`;
                if ($?>0) {
                push(@errors,"Problem running $survsupr to initialize dialer ($!)\n$supr_out\n");
                &get_out(@errors);
                } else {
                print "Dialer $num Initialized\n";
                unlink $suprspec;
                }

   return;
}



sub get_out {
  my @errors = @_;
  print "The following errors will keep us from continuing:\n";
  foreach my $err(@errors) {
    print "  $err\n";
  }
  exit(scalar(@errors));
}

sub check_ex {
  my @progs = @_;
  my @errors = ();
  foreach my $prog(@progs) {
    if (! -e $prog) {
      push(@errors,"$prog does not exist");
    }
    elsif (! -x $prog) {
      push(@errors,"$prog is not executable");
    }
  }
  return @errors;
}

sub check_env_var {
   my ( $v, $type, $rwx ) = @_;
   my $var = $ENV{$v};
   my @errors = ();
   my %alltypes = (file => 1, directory => 1, string => 1, integer => 1);
   unless (exists($alltypes{$type})) {
     push(@errors,"Invalid type in check_env_var(): $type");
   }
   if ( $var ) {
     if ($type eq "file" or $type eq "directory") {
       &check_access($var,$type,$rwx);
     } elsif ($type eq "integer") {
       unless ($var =~ /^[-]?\d+$/ ) {
         push(@errors, "Environment Variable $v ($var) is a string or non-integer");
       }
     }
   } else {
     push(@errors,"Environment Variable $v is not defined");
   }
   return @errors;
}

sub check_access {
   my ( $name, $type, $rwx ) = @_;
   my @errors = ();
   my %alltypes = ("file" => 1, "directory" => 1);
   unless (exists($alltypes{$type})) {
     push(@errors,"Invalid type in check_access(): $type");
   }
   if ( -e $name ) {
     if ( -d $name ) {
       push(@errors,"$name is a directory") if ($type eq "file");
     } else {
       push(@errors,"$name is not a directory") if ($type eq "directory");
     }
     if (index($rwx,"r")>=0) {
       if (! -r $name ) {
         push(@errors,"$name is not readable");
       }
     }
     if (index($rwx,"w")>=0) {
       if ( ! -w $name ) {
         push(@errors,"$name is not writable to");
       }
     }
     if (index($rwx,"x")>=0) {
       if ( ! -x $name ) {
         push(@errors,"$name is not executable");
       }
     }
   } else {
     push(@errors,"$name does not exist");
   }
   return @errors;
}

sub get_ps_id {
  my($proc)=@_;
  my $psopt="-Af";
  my $ps_id = "";
  if ($proc) {
    chomp(my $ps_list = `ps $psopt | grep '$proc' | grep -v grep`);
    if ($ps_list) {
      my @ps_items=split(/\s+/,$ps_list);
      $ps_id = $ps_items[1];
    }
  }
  return $ps_id;
}

sub check_makecfg {
   my ( $prog, $ldev ) = @_;
   my @errors = ();
   my $in     = "/tmp/s1$$.spx";
   open( IN, ">$in" )
     or push( @errors, "Cannot create $in to check for study server\n($!)" );
   print IN "find server\nquit\n";
   close(IN);
   my $cfgout = `$prog $in con`;
   push( @errors,
      "Problem running $prog to check for running study server\n$cfgout\n" )
     if ( $? > 0 );
   unlink($in);

   # NO SERVER FOUND:
   #   *** No active (c-server) found ***

   #   LDEV: 0
   # SERVER FOUND:
   #   c-server env: /cfmc/test8.1/ipcfiles/
   #   LDEV: 9901
   if ( index( $cfgout, "*** No active (c-server) found ***" ) < 0 ) {
      my @cfglst = split( /\n/, $cfgout );
      foreach my $c (@cfglst) {

         # (index($c,"c-server env: $ENV{CFMCCFG}")>=0)
         if ( index( $c, "LDEV: " ) >= 0 ) {
            $ldev = $c;
            $ldev =~ s/LDEV: //g;
            last;
         }
      }
   }
   else {
      $ldev = 0;
   }
   return $ldev, @errors;
}

#--------------------------------------------------------------------------
# Get the next available unused LDEV
# input: (makecfg program ($mcfg), initial LDEV ($LDEV)
# output: return LDEV to use ($LDEV), SOCKET to use ($SOCKET), array of error messages (@erro$
#--------------------------------------------------------------------------

sub nextldev
{
    my ( $mcfg, $LDEV ) = @_;
    my @errors  = ();
    my $SOCKET  = 0;
    my $ldevmax = 9999;         # maximum ldev #
    my $sockadd = 10000;        # what to add to ldev to get socket
    my $in      = "/tmp/cf$$.spx";
    open( IN, ">$in" ) or push( @errors, "Cannot open $in ($!)" );
    print IN "show stations\n";
    print IN "quit\n";
    close(IN);
    my $out = `$mcfg $in con`;
    if ( $? > 0 ) {
        push( @errors, "Problem running $mcfg to get available ldev ($!)\n$out" );
    }
    else {
        my @lines = split( /\n/, $out );
        foreach my $l (@lines) {
            my @line = split( /\s+/, $l );
            if ( scalar(@line) > 0 and $LDEV eq $line[0] ) {
                $LDEV = $LDEV + 1;
                push( @errors, "We've reached the maximum LDEV number of $ldevmax, please try a lower initial LDEV" )
                  if ( $LDEV >= $ldevmax );
            }
        }
    }
    unlink($in);
    $SOCKET = $LDEV + $sockadd;
  return ( $LDEV, $SOCKET, \@errors );
} ## end sub nextldev

#--------------------------------------------------------------------------
# slurp a file into a single scalar or a line-by-line array
#--------------------------------------------------------------------------
sub read_file {
   my $text = "";

   $text = do { local( @ARGV, $/ ) = $_[0] ; <> } if (-f $_[0]);

   # return handles call from different contexts
   return undef unless defined wantarray;      # called in void context
   return split /\n/, $text if wantarray;      # called in list context
   return $text;                               # called in scalar context
}
